#include <stdio.h>
#include "geoipmax_amxx.h"

GeoIP *gi = NULL;
GeoIP *giCity = NULL;

static const char *mark_NA(const char *p)
{
	return p ? p : "N/A";
}
inline void StripPort(char *ip)
{
	char *tmp = strchr(ip, ':');
	if (!tmp)
		return;
	*tmp = '\0';
}

static cell AMX_NATIVE_CALL amx_geoip_code2(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);
	const char *ccode = GeoIP_country_code_by_addr(gi, ip);
	return MF_SetAmxString(amx, params[2], ccode?ccode:"error", 3);
}

static cell AMX_NATIVE_CALL amx_geoip_code3(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);
	const char *ccode = GeoIP_country_code3_by_addr(gi, ip);
	return MF_SetAmxString(amx, params[2], ccode?ccode:"error", 4);
}
static cell AMX_NATIVE_CALL amx_geoip_country(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);
	const char *ccode = GeoIP_country_name_by_addr(gi, ip);
	return MF_SetAmxString(amx, params[2], ccode?ccode:"error", params[3]);
}

static cell AMX_NATIVE_CALL amx_geoip_country_code2(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	const char *ccode = gi ? GeoIP_country_code_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[2], ccode ? ccode : "--", 2);
	return ccode != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_country_code3(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	const char *ccode = gi ? GeoIP_country_code3_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[2], ccode ? ccode : "--", 3);
	return ccode != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_country_name(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	const char *cname = gi ? GeoIP_country_name_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[2], mark_NA(cname), params[3]);
	return cname != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_country_name_by_cc(AMX *amx, cell *params)
{
	int len = 0;
	char *ccode = MF_GetAmxString(amx, params[1], 0, &len);

	int id = GeoIP_id_by_code(ccode);
	const char *cname = GeoIP_name_by_id(id);
	MF_SetAmxString(amx, params[2], mark_NA(cname), params[3]);
	return cname != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_region_name(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	GeoIPRecord *gir = giCity ? GeoIP_record_by_addr(giCity, ip) : NULL;
	const char *region = gir ? GeoIP_region_name_by_code(gir->country_code, gir->region) : NULL;
	MF_SetAmxString(amx, params[2], mark_NA(region), params[3]);
	if (gir != NULL)
		GeoIPRecord_delete(gir);
	return region != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_city_name(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	GeoIPRecord *gir = giCity ? GeoIP_record_by_addr(giCity, ip) : NULL;
	const char *city = gir ? gir->city : NULL;
	MF_SetAmxString(amx, params[2], mark_NA(city), params[3]);
	if (gir != NULL)
		GeoIPRecord_delete(gir);
	return city != NULL;
}

static cell AMX_NATIVE_CALL amx_geoip_coords(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	GeoIPRecord *gir = giCity ? GeoIP_record_by_addr(giCity, ip) : NULL;
	cell *latByRef = MF_GetAmxAddr(amx, params[2]);
	cell *lngByRef = MF_GetAmxAddr(amx, params[3]);
	float lat = gir ? gir->latitude : 0;
	float lng = gir ? gir->longitude : 0;
	*latByRef = amx_ftoc(lat);
	*lngByRef = amx_ftoc(lng);
	if (gir != NULL)
		GeoIPRecord_delete(gir);
	return lat != 0 && lng != 0;
}

static cell AMX_NATIVE_CALL amx_geoip_range(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	bool success = true;
	char **range = giCity ? GeoIP_range_by_ip(giCity, ip) : NULL;
	if (range != NULL)
	{
		MF_SetAmxString(amx, params[2], range[0] ? range[0] : "", params[3]);
		MF_SetAmxString(amx, params[4], range[1] ? range[1] : "", params[5]);
		if (range[0] == NULL || range[1] == NULL) success = false;
		GeoIP_range_by_ip_delete(range);
	}
	else
	{
		MF_SetAmxString(amx, params[2], "", params[3]);
		MF_SetAmxString(amx, params[4], "", params[5]);
		success = false;
	}
	return success;
}

static cell AMX_NATIVE_CALL amx_geoip_fullinfo(AMX *amx, cell *params)
{
	int len = 0;
	char *ip = MF_GetAmxString(amx, params[1], 0, &len);
	StripPort(ip);

	bool success = true;

	const char *ccode = gi ? GeoIP_country_code_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[2], ccode ? ccode : "--", 2);
	if (ccode == NULL) success = false;

	ccode = gi ? GeoIP_country_code3_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[3], ccode ? ccode : "--", 3);
	if (ccode == NULL) success = false;

	const char *cname = gi ? GeoIP_country_name_by_addr(gi, ip) : NULL;
	MF_SetAmxString(amx, params[4], mark_NA(cname), params[5]);
	if (cname == NULL) success = false;



	GeoIPRecord *gir = giCity ? GeoIP_record_by_addr(giCity, ip) : NULL;

	const char *region = gir ? GeoIP_region_name_by_code(gir->country_code, gir->region) : NULL;
	MF_SetAmxString(amx, params[6], mark_NA(region), params[7]);
	if (region == NULL) success = false;

	const char *city = gir ? gir->city : NULL;
	MF_SetAmxString(amx, params[8], mark_NA(city), params[9]);
	if (city == NULL) success = false;

	cell *latByRef = MF_GetAmxAddr(amx, params[10]);
	cell *lngByRef = MF_GetAmxAddr(amx, params[11]);
	float lat = gir ? gir->latitude : 0;
	float lng = gir ? gir->longitude : 0;
	*latByRef = amx_ftoc(lat);
	*lngByRef = amx_ftoc(lng);

	if (gir != NULL)
		GeoIPRecord_delete(gir);



	char **range = giCity ? GeoIP_range_by_ip(giCity, ip) : NULL;
	if (range != NULL)
	{
		MF_SetAmxString(amx, params[12], range[0] ? range[0] : "", params[13]);
		MF_SetAmxString(amx, params[14], range[1] ? range[1] : "", params[15]);
		if (range[0] == NULL || range[1] == NULL) success = false;
		GeoIP_range_by_ip_delete(range);
	}
	else
	{
		MF_SetAmxString(amx, params[12], "", params[13]);
		MF_SetAmxString(amx, params[14], "", params[15]);
		success = false;
	}

	return success;
}

void OnAmxxAttach()
{
	char *path = MF_BuildPathname("%s/GeoIP.dat",MF_GetLocalInfo("amxx_datadir", "addons/amxmodx/data"));
	gi = GeoIP_open(path, GEOIP_MEMORY_CACHE);
	if (gi == NULL)
	{
		MF_Log("Failed to instantiate GeoIP!");
		//return;
	}
	
	path = MF_BuildPathname("%s/GeoLiteCity.dat", MF_GetLocalInfo("amxx_datadir", "addons/amxmodx/data"));
	giCity = GeoIP_open(path, GEOIP_INDEX_CACHE);
	if (giCity == NULL)
	{
		MF_Log("Failed to instantiate GeoLiteCity!");
		//return;
	}

	char *db_info = GeoIP_database_info(gi);
	MF_Log("Database info: %s", db_info);
	db_info = GeoIP_database_info(giCity);
	MF_Log("Database info: %s", db_info);
	MF_AddNatives(geoip_natives);
}

void OnAmxxDetach()
{
	GeoIP_delete(gi);
	gi = NULL;
	GeoIP_delete(giCity);
	giCity = NULL;
}

AMX_NATIVE_INFO geoip_natives[] = {
	{"geoip_code2",					amx_geoip_code2},
	{"geoip_code3",					amx_geoip_code3},
	{"geoip_country",				amx_geoip_country},

	{"geoip_code2_ex",				amx_geoip_country_code2},
	{"geoip_code3_ex",				amx_geoip_country_code3},

	{"geoip_country_code2",			amx_geoip_country_code2},
	{"geoip_country_code3",			amx_geoip_country_code3},
	{"geoip_country_name",			amx_geoip_country_name},
	{"geoip_country_name_by_cc",	amx_geoip_country_name_by_cc},
	{"geoip_region_name",			amx_geoip_region_name},
	{"geoip_city_name",				amx_geoip_city_name},
	{"geoip_coords",				amx_geoip_coords},
	{"geoip_range",					amx_geoip_range},
	{"geoip_fullinfo",				amx_geoip_fullinfo},

	{NULL,							NULL},
};
